@@ -4,13 +4,13 @@ require 'tempfile'
 benchmark_dir = File.expand_path(File.dirname(__FILE__))
 top_dir = File.expand_path(File.join(benchmark_dir, ".."))
 test_dir = File.join(top_dir, "test")
-$LOAD_PATH.unshift(File.join(top_dir, "ext", "ruby", ".libs"))
-$LOAD_PATH.unshift(File.join(top_dir, "ext", "ruby"))
+$LOAD_PATH.unshift(File.join(top_dir, "bindings", "ruby", ".libs"))
+$LOAD_PATH.unshift(File.join(top_dir, "bindings", "ruby"))
 $LOAD_PATH.unshift(File.join(test_dir))
 
-require 'tomoe-spec-utils'
+require 'tomoe-test-utils'
 
-data_dir = TomoeSpecUtils::Path.data_dir
+data_dir = TomoeTestUtils::Path.data_dir
 use_est = false
 if use_est
   database = File.join(data_dir, "handwriting")
