@@ -17,15 +17,19 @@
  *  Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  *  Boston, MA  02111-1307  USA
  *
- *  $Id: glib-utils.h,v 1.9 2006/12/25 03:32:50 kous Exp $
+ *  $Id: glib-utils.h 1212 2007-04-25 07:35:56Z makeinu $
  */
 
 
 #ifndef __GLIB_UTILS_H__
 #define __GLIB_UTILS_H__
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif /* HAVE_CONFIG_H */
+
 #include <glib/garray.h>
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include "glib-compat-file-utilities.h"
 #include "glib-compat-key-file.h"
 
