@@ -18,13 +18,17 @@
  *  Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  *  Boston, MA  02111-1307  USA
  *
- *  $Id: tomoe-char.c,v 1.62 2006/12/22 05:26:03 kous Exp $
+ *  $Id: tomoe-char.c 1488 2007-06-16 01:16:23Z ikezoe $
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif /* HAVE_CONFIG_H */
+
 #include <stdlib.h>
 #include <string.h>
 #include <glib.h>
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 
 #include "tomoe-char.h"
 #include "tomoe-dict.h"
@@ -56,15 +60,15 @@ enum
 
 G_DEFINE_TYPE (TomoeChar, tomoe_char, G_TYPE_OBJECT)
 
-static void tomoe_char_dispose        (GObject         *object);
-static void tomoe_char_set_property   (GObject         *object,
-                                       guint            prop_id,
-                                       const GValue    *value,
-                                       GParamSpec      *pspec);
-static void tomoe_char_get_property   (GObject         *object,
-                                       guint            prop_id,
-                                       GValue          *value,
-                                       GParamSpec      *pspec);
+static void dispose        (GObject         *object);
+static void set_property   (GObject         *object,
+                            guint            prop_id,
+                            const GValue    *value,
+                            GParamSpec      *pspec);
+static void get_property   (GObject         *object,
+                            guint            prop_id,
+                            GValue          *value,
+                            GParamSpec      *pspec);
 
 static void
 tomoe_char_class_init (TomoeCharClass *klass)
@@ -74,9 +78,9 @@ tomoe_char_class_init (TomoeCharClass *klass)
 
     gobject_class = G_OBJECT_CLASS (klass);
 
-    gobject_class->dispose      = tomoe_char_dispose;
-    gobject_class->set_property = tomoe_char_set_property;
-    gobject_class->get_property = tomoe_char_get_property;
+    gobject_class->dispose      = dispose;
+    gobject_class->set_property = set_property;
+    gobject_class->get_property = get_property;
 
     spec = g_param_spec_string ("utf8",
                                 N_("UTF8"),
@@ -143,7 +147,61 @@ tomoe_char_new_from_xml_data (const gchar *data, gssize len)
 }
 
 static void
-tomoe_char_dispose (GObject *object)
+copy_meta_data (gpointer key, gpointer value, gpointer user_data)
+{
+    TomoeChar *chr = user_data;
+    gchar *meta_key = key;
+    gchar *meta_value = value;
+
+    tomoe_char_register_meta_data (chr, meta_key, meta_value);
+}
+
+TomoeChar*
+tomoe_char_dup (TomoeChar *chr)
+{
+    TomoeChar *new_chr;
+    TomoeCharPrivate *priv;
+
+    new_chr = tomoe_char_new ();
+    priv = TOMOE_CHAR_GET_PRIVATE (chr);
+
+    tomoe_char_set_utf8 (new_chr, priv->utf8);
+    tomoe_char_set_n_strokes (new_chr, priv->n_strokes);
+
+    if (priv->writing) {
+        TomoeWriting *new_writing = tomoe_writing_dup (priv->writing);
+        tomoe_char_set_writing (new_chr, new_writing);
+        g_object_unref (new_writing);
+    }
+
+    if (priv->variant)
+        tomoe_char_set_variant (new_chr, priv->variant);
+
+    if (priv->readings) {
+        GList *node; 
+        for (node = g_list_last(priv->readings); node; node = g_list_previous (node)) {
+            TomoeReading *new_reading = tomoe_reading_dup (TOMOE_READING(node->data));
+            tomoe_char_add_reading (new_chr, new_reading);
+            g_object_unref (new_reading);
+        }
+    }
+
+    if (priv->radicals) {
+        GList *node; 
+        for (node = g_list_last(priv->radicals); node; node = g_list_previous (node)) {
+            tomoe_char_add_radical (new_chr, node->data);
+        }
+    }
+
+    if (priv->meta_data) {
+        tomoe_char_meta_data_foreach (chr, (GHFunc) copy_meta_data, new_chr);
+    }
+
+    return new_chr;
+}
+
+static void
+dispose (GObject *object)
 {
     TomoeCharPrivate *priv = TOMOE_CHAR_GET_PRIVATE (object);
 
@@ -176,10 +234,10 @@ tomoe_char_dispose (GObject *object)
 }
 
 static void
-tomoe_char_set_property (GObject      *object,
-                         guint         prop_id,
-                         const GValue *value,
-                         GParamSpec   *pspec)
+set_property (GObject      *object,
+              guint         prop_id,
+              const GValue *value,
+              GParamSpec   *pspec)
 {
     TomoeChar *chr;
 
@@ -204,10 +262,10 @@ tomoe_char_set_property (GObject      *object,
 }
 
 static void
-tomoe_char_get_property (GObject    *object,
-                         guint       prop_id,
-                         GValue     *value,
-                         GParamSpec *pspec)
+get_property (GObject    *object,
+              guint       prop_id,
+              GValue     *value,
+              GParamSpec *pspec)
 {
     TomoeChar *chr;
     TomoeCharPrivate *priv;
@@ -253,7 +311,8 @@ tomoe_char_set_utf8 (TomoeChar *chr, const gchar *utf8)
     g_return_if_fail (TOMOE_IS_CHAR (chr));
 
     priv = TOMOE_CHAR_GET_PRIVATE (chr);
-    g_free (priv->utf8);
+    if (priv->utf8)
+        g_free (priv->utf8);
     priv->utf8 = utf8 ? g_strdup (utf8) : NULL;
 }
 
@@ -483,7 +542,7 @@ tomoe_char_to_xml_readings (TomoeChar *chr, TomoeCharPrivate *priv,
     if (!priv->readings) return;
 
     g_string_append (output, "    <readings>\n");
-    for (node = priv->readings; node; node = g_list_next (node)) {
+    for (node = g_list_last (priv->readings); node; node = g_list_previous (node)) {
         TomoeReading *reading = node->data;
         gchar *xml;
 
@@ -532,6 +591,18 @@ tomoe_char_to_xml_writing (TomoeChar *chr, TomoeCharPrivate *priv,
     }
 }
 
+static void
+tomoe_char_to_xml_n_strokes (TomoeChar *chr, TomoeCharPrivate *priv,
+                             GString *output)
+{
+    if (priv->n_strokes < 0) return;
+
+    g_string_append_printf (
+        output,
+        "    <number-of-strokes>%d</number-of-strokes>\n",
+        priv->n_strokes);
+}
+
 
 static void
 tomoe_char_to_xml_meta_datum (gpointer key, gpointer value, gpointer user_data)
@@ -572,6 +643,7 @@ tomoe_char_to_xml (TomoeChar* chr)
     tomoe_char_to_xml_variant (chr, priv, output);
     tomoe_char_to_xml_readings (chr, priv, output);
     tomoe_char_to_xml_radicals (chr, priv, output);
+    tomoe_char_to_xml_n_strokes (chr, priv, output);
     tomoe_char_to_xml_writing (chr, priv, output);
     tomoe_char_to_xml_meta (chr, priv, output);
 
