@@ -17,7 +17,7 @@
  *  Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  *  Boston, MA  02111-1307  USA
  *
- *  $Id: tomoe-config.c,v 1.56 2006/12/21 02:55:11 kous Exp $
+ *  $Id: tomoe-config.c 1307 2007-06-07 03:31:25Z ikezoe $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -28,7 +28,7 @@
 #include <string.h>
 #include <sys/stat.h>
 #include <glib.h>
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 
 #include "tomoe-config.h"
 #include "tomoe-dict.h"
@@ -46,6 +46,7 @@ struct _TomoeConfigPrivate
     gchar       *filename;
     gchar       *user_dict_name;
     GKeyFile    *key_file;
+    gchar		**languages;
 };
 
 enum
@@ -59,15 +60,18 @@ G_DEFINE_TYPE (TomoeConfig, tomoe_config, G_TYPE_OBJECT)
 
 static const gchar *system_config_file = CONFDIR "/config";
 
-static void     tomoe_config_dispose      (GObject       *object);
-static void     tomoe_config_set_property (GObject       *object,
-                                           guint          prop_id,
-                                           const GValue  *value,
-                                           GParamSpec    *pspec);
-static void     tomoe_config_get_property (GObject       *object,
-                                           guint          prop_id,
-                                           GValue        *value,
-                                           GParamSpec    *pspec);
+static void     dispose      (GObject       *object);
+static GObject *constructor  (GType                  type,
+                              guint                  n_props,
+                              GObjectConstructParam *props);
+static void     set_property (GObject       *object,
+                              guint          prop_id,
+                              const GValue  *value,
+                              GParamSpec    *pspec);
+static void     get_property (GObject       *object,
+                              guint          prop_id,
+                              GValue        *value,
+                              GParamSpec    *pspec);
 
 static void     tomoe_config_load         (TomoeConfig  *config);
 static void     tomoe_config_save         (TomoeConfig *config);
@@ -80,6 +84,10 @@ static gchar   *_tomoe_config_key_file_get_string  (GKeyFile *key_file,
                                                     const gchar *group,
                                                     const gchar *key,
                                                     const gchar *default_value);
+static gchar  **_tomoe_config_key_file_get_string_list(GKeyFile *key_file,
+                                                       const gchar *group,
+                                                       const gchar *key,
+                                                       gchar **default_value);
 static gint     _tomoe_config_key_file_get_integer (GKeyFile *key_file,
                                                     const gchar *group,
                                                     const gchar *key,
@@ -97,21 +105,36 @@ tomoe_config_class_init (TomoeConfigClass *klass)
 
     gobject_class = G_OBJECT_CLASS (klass);
 
-    gobject_class->dispose      = tomoe_config_dispose;
-    gobject_class->set_property = tomoe_config_set_property;
-    gobject_class->get_property = tomoe_config_get_property;
+    gobject_class->dispose      = dispose;
+    gobject_class->constructor  = constructor;
+    gobject_class->set_property = set_property;
+    gobject_class->get_property = get_property;
 
     g_object_class_install_property (gobject_class,
                                      PROP_FILENAME,
                                      g_param_spec_string ("filename",
                                          N_("Filename"),
-                                         N_("The Filename for storing user settings"),
+                                         N_("The filename for storing user settings"),
                                          NULL,
                                          G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
 
     g_type_class_add_private (gobject_class, sizeof (TomoeConfigPrivate));
 }
 
+static GObject *
+constructor (GType type, guint n_props, GObjectConstructParam *props)
+{
+
+    GObject *object;
+    GObjectClass *klass = G_OBJECT_CLASS (tomoe_config_parent_class);
+
+    object = klass->constructor (type, n_props, props);
+
+    tomoe_config_load (TOMOE_CONFIG (object));
+
+    return object;
+}
+
 static void
 tomoe_config_init (TomoeConfig *config)
 {
@@ -120,6 +143,7 @@ tomoe_config_init (TomoeConfig *config)
     priv->filename   = NULL;
     priv->user_dict_name = NULL;
     priv->key_file   = NULL;
+    priv->languages  = NULL;
 }
 
 TomoeConfig *
@@ -131,13 +155,11 @@ tomoe_config_new (const gchar *config_file)
                           "filename", config_file,
                           NULL);
 
-    tomoe_config_load (config);
-
     return config;
 }
 
 static void
-tomoe_config_dispose (GObject *object)
+dispose (GObject *object)
 {
     TomoeConfig *config;
     TomoeConfigPrivate *priv;
@@ -153,25 +175,30 @@ tomoe_config_dispose (GObject *object)
         g_free (priv->user_dict_name);
     if (priv->key_file)
         g_key_file_free (priv->key_file);
+    if (priv->languages)
+        g_strfreev (priv->languages);
 
     priv->filename  = NULL;
     priv->user_dict_name = NULL;
     priv->key_file = NULL;
+    priv->languages = NULL;
 
     if (G_OBJECT_CLASS (tomoe_config_parent_class)->dispose)
         G_OBJECT_CLASS (tomoe_config_parent_class)->dispose (object);
 }
 
 static void
-tomoe_config_set_property (GObject      *object,
-                           guint         prop_id,
-                           const GValue *value,
-                           GParamSpec   *pspec)
+set_property (GObject      *object,
+              guint         prop_id,
+              const GValue *value,
+              GParamSpec   *pspec)
 {
     TomoeConfigPrivate *priv = TOMOE_CONFIG_GET_PRIVATE (object);
 
     switch (prop_id) {
       case PROP_FILENAME:
+        if (priv->filename)
+            g_free (priv->filename);
         priv->filename = g_value_dup_string (value);
         break;
 
@@ -182,10 +209,10 @@ tomoe_config_set_property (GObject      *object,
 }
 
 static void
-tomoe_config_get_property (GObject    *object,
-                           guint       prop_id,
-                           GValue     *value,
-                           GParamSpec *pspec)
+get_property (GObject    *object,
+              guint       prop_id,
+              GValue     *value,
+              GParamSpec *pspec)
 {
     TomoeConfigPrivate *priv = TOMOE_CONFIG_GET_PRIVATE (object);
 
@@ -207,6 +234,7 @@ tomoe_config_load (TomoeConfig *config)
     GError *error = NULL;
     TomoeConfigPrivate *priv;
     const gchar *config_file;
+    gchar *language;
 
     g_return_if_fail (config);
 
@@ -230,10 +258,27 @@ tomoe_config_load (TomoeConfig *config)
 
     priv->key_file = key_file;
 
+    if (priv->user_dict_name)
+        g_free (priv->user_dict_name);
     priv->user_dict_name =
         _tomoe_config_key_file_get_string (key_file,
                                            "config", "user-dictionary",
                                            DEFAULT_USER_DICT_NAME);
+
+    if (priv->languages)
+        g_strfreev (priv->languages);
+    language = _tomoe_config_key_file_get_string (key_file,
+                                                  "config", "language",
+                                                  NULL);
+    if (language) {
+        priv->languages = g_new0(gchar *, 2);
+        priv->languages[0] = language;
+    } else {
+        priv->languages =
+            _tomoe_config_key_file_get_string_list (key_file,
+                                                    "config", "languages",
+                                                    NULL);
+    }
 }
 
 static void
@@ -285,7 +330,7 @@ tomoe_config_get_user_dict_name (TomoeConfig *config)
 }
 
 TomoeShelf *
-tomoe_config_make_shelf (TomoeConfig *config)
+tomoe_config_make_shelf (TomoeConfig *config, const gchar *language)
 {
     TomoeConfigPrivate *priv;
     TomoeShelf *shelf;
@@ -339,6 +384,18 @@ tomoe_config_make_shelf (TomoeConfig *config)
     return shelf;
 }
 
+const gchar *const *
+tomoe_config_get_languages (TomoeConfig *config)
+{
+    TomoeConfigPrivate *priv;
+
+    g_return_val_if_fail (TOMOE_IS_CONFIG (config), NULL);
+
+    priv = TOMOE_CONFIG_GET_PRIVATE (config);
+    return (const gchar *const *)(priv->languages);
+}
+
+
 static gboolean
 _tomoe_config_key_file_get_boolean (GKeyFile *key_file,
                                     const gchar *group,
@@ -390,6 +447,33 @@ _tomoe_config_key_file_get_string (GKeyFile *key_file,
     return result;
 }
 
+static gchar **
+_tomoe_config_key_file_get_string_list (GKeyFile *key_file,
+                                        const gchar *group,
+                                        const gchar *key,
+                                        gchar **default_value)
+{
+    gchar **result = NULL;
+    gsize length = 0;
+    GError *error = NULL;
+
+    result = g_key_file_get_string_list (key_file, group, key, &length, &error);
+    if (error) {
+        switch (error->code) {
+          case G_KEY_FILE_ERROR_NOT_FOUND:
+            g_error_free (error);
+            break;
+          case G_KEY_FILE_ERROR_INVALID_VALUE:
+            TOMOE_HANDLE_ERROR (error);
+            break;
+        }
+        if (default_value)
+            result = g_strdupv (default_value);
+    }
+
+    return result;
+}
+
 static gint
 _tomoe_config_key_file_get_integer (GKeyFile *key_file,
                                     const gchar *group,
