@@ -17,13 +17,17 @@
  *  Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  *  Boston, MA  02111-1307  USA
  *
- *  $Id: tomoe-reading.c,v 1.3 2006/11/30 09:13:15 kous Exp $
+ *  $Id: tomoe-reading.c 1479 2007-06-15 07:03:06Z ikezoe $
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif /* HAVE_CONFIG_H */
+
 #include <stdlib.h>
 #include <string.h>
 #include <glib.h>
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 
 #include "tomoe-enum-types.h"
 #include "tomoe-reading.h"
@@ -48,15 +52,15 @@ enum
 
 G_DEFINE_TYPE (TomoeReading, tomoe_reading, G_TYPE_OBJECT)
 
-static void tomoe_reading_dispose        (GObject         *object);
-static void tomoe_reading_set_property   (GObject         *object,
-                                          guint            prop_id,
-                                          const GValue    *value,
-                                          GParamSpec      *pspec);
-static void tomoe_reading_get_property   (GObject         *object,
-                                          guint            prop_id,
-                                          GValue          *value,
-                                          GParamSpec      *pspec);
+static void dispose        (GObject         *object);
+static void set_property   (GObject         *object,
+                            guint            prop_id,
+                            const GValue    *value,
+                            GParamSpec      *pspec);
+static void get_property   (GObject         *object,
+                            guint            prop_id,
+                            GValue          *value,
+                            GParamSpec      *pspec);
 
 static void
 tomoe_reading_class_init (TomoeReadingClass *klass)
@@ -66,9 +70,9 @@ tomoe_reading_class_init (TomoeReadingClass *klass)
 
     gobject_class = G_OBJECT_CLASS (klass);
 
-    gobject_class->dispose  = tomoe_reading_dispose;
-    gobject_class->set_property = tomoe_reading_set_property;
-    gobject_class->get_property = tomoe_reading_get_property;
+    gobject_class->dispose      = dispose;
+    gobject_class->set_property = set_property;
+    gobject_class->get_property = get_property;
 
     spec = g_param_spec_enum ("type",
                               N_("Type"),
@@ -107,8 +111,20 @@ tomoe_reading_new (TomoeReadingType type, const gchar *reading)
                         NULL);
 }
 
+TomoeReading *
+tomoe_reading_dup (TomoeReading *reading)
+{
+    TomoeReadingPrivate *priv;
+
+    g_return_val_if_fail (TOMOE_IS_READING (reading), NULL);
+
+    priv = TOMOE_READING_GET_PRIVATE (reading);
+
+    return tomoe_reading_new (priv->type, priv->reading);
+}
+
 static void
-tomoe_reading_dispose (GObject *object)
+dispose (GObject *object)
 {
     TomoeReadingPrivate *priv = TOMOE_READING_GET_PRIVATE (object);
 
@@ -119,10 +135,10 @@ tomoe_reading_dispose (GObject *object)
     G_OBJECT_CLASS (tomoe_reading_parent_class)->dispose (object);
 }
 static void
-tomoe_reading_set_property (GObject      *object,
-                            guint         prop_id,
-                            const GValue *value,
-                            GParamSpec   *pspec)
+set_property (GObject      *object,
+              guint         prop_id,
+              const GValue *value,
+              GParamSpec   *pspec)
 {
     TomoeReading *reading;
     TomoeReadingPrivate *priv;
@@ -145,10 +161,10 @@ tomoe_reading_set_property (GObject      *object,
 }
 
 static void
-tomoe_reading_get_property (GObject    *object,
-                            guint       prop_id,
-                            GValue     *value,
-                            GParamSpec *pspec)
+get_property (GObject    *object,
+              guint       prop_id,
+              GValue     *value,
+              GParamSpec *pspec)
 {
     TomoeReading *reading;
     TomoeReadingPrivate *priv;
@@ -227,6 +243,59 @@ tomoe_reading_to_xml (TomoeReading *reading)
         return g_markup_printf_escaped (format, priv->reading);
 }
 
+/**
+ * tomoe_reading_compare:
+ * @a: a TomoeReading object.
+ * @b: a TomoeReading object to compare with.
+ *
+ * Compare to TomoeReading objects with its own utf8 character.
+ *
+ * Return value: -1 a != b, 0 a= b, 
+ */
+gint
+tomoe_reading_compare (const TomoeReading *a, const TomoeReading *b)
+{
+    TomoeReadingPrivate *priv_a, *priv_b;
+
+    if (!a || !b) return 0;
+
+    priv_a = TOMOE_READING_GET_PRIVATE (a);
+    priv_b = TOMOE_READING_GET_PRIVATE (b);
+    if (!priv_a || !priv_b) return 0;
+
+    if (!priv_a->reading || !priv_b->reading) return 0;
+
+    if (priv_a->type != priv_b->type &&
+        (priv_a->type != TOMOE_READING_UNKNOWN &&
+         priv_b->type != TOMOE_READING_UNKNOWN)) {
+        return -1;
+    }
+
+    return strcmp (priv_a->reading, priv_b->reading);
+}
+
+gboolean
+tomoe_reading_has_prefix (TomoeReading *reading, const TomoeReading *prefix_reading)
+{
+    TomoeReadingPrivate *priv_a, *priv_b;
+
+    if (!reading || !prefix_reading) return TRUE;
+
+    priv_a = TOMOE_READING_GET_PRIVATE (reading);
+    priv_b = TOMOE_READING_GET_PRIVATE (prefix_reading);
+    if (!priv_a || !priv_b) return 0;
+
+    if (!priv_a->reading || !priv_b->reading) return TRUE;
+
+    if (priv_a->type != priv_b->type &&
+        (priv_a->type != TOMOE_READING_UNKNOWN &&
+         priv_b->type != TOMOE_READING_UNKNOWN)) {
+        return FALSE;
+    }
+
+    return g_str_has_prefix (priv_a->reading, priv_b->reading);
+}
+
 /*
 vi:ts=4:nowrap:ai:expandtab
 */
