@@ -18,7 +18,7 @@
  *  Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  *  Boston, MA  02111-1307  USA
  *
- *  $Id: tomoe-writing.c,v 1.13 2006/12/18 02:12:49 kous Exp $
+ *  $Id: tomoe-writing.c 1476 2007-06-15 06:39:14Z ikezoe $
  */
 
 #include <stdlib.h>
@@ -39,7 +39,7 @@ struct _TomoeWritingPrivate
 
 G_DEFINE_TYPE (TomoeWriting, tomoe_writing, G_TYPE_OBJECT)
 
-static void tomoe_writing_dispose (GObject *object);
+static void          dispose      (GObject *object);
 static GList       *_stroke_new   (gint x, gint y);
 static void         _stroke_free  (GList *stroke);
 
@@ -50,7 +50,7 @@ tomoe_writing_class_init (TomoeWritingClass *klass)
 
     gobject_class = G_OBJECT_CLASS (klass);
 
-    gobject_class->dispose = tomoe_writing_dispose;
+    gobject_class->dispose = dispose;
 
     g_type_class_add_private (gobject_class, sizeof (TomoeWritingPrivate));
 }
@@ -64,7 +64,7 @@ tomoe_writing_init (TomoeWriting *writing)
 }
 
 static void
-tomoe_writing_dispose (GObject *object)
+dispose (GObject *object)
 {
     TomoeWriting *writing = TOMOE_WRITING (object);
 
@@ -80,6 +80,43 @@ tomoe_writing_new (void)
     return writing;
 }
 
+TomoeWriting *
+tomoe_writing_dup (TomoeWriting *writing)
+{
+    TomoeWriting *new_writing;
+    TomoeWritingPrivate *priv;
+    GList *stroke_list;
+
+    g_return_val_if_fail (TOMOE_IS_WRITING (writing), NULL);
+
+    new_writing = tomoe_writing_new ();
+
+    priv = TOMOE_WRITING_GET_PRIVATE(writing);
+    if (!priv->stroke_first) return new_writing;
+
+    for (stroke_list = priv->stroke_first;
+         stroke_list;
+         stroke_list = g_list_next (stroke_list)) {
+        GList *point_list = stroke_list->data;
+        GList *first_point = point_list;
+
+        if (!point_list) continue;
+
+        for (; point_list; point_list = g_list_next (point_list)) {
+            TomoePoint *p = point_list->data;
+
+            if (!p) continue;
+ 
+            if (point_list == first_point)
+                tomoe_writing_move_to (new_writing, p->x, p->y);
+            else
+                tomoe_writing_line_to (new_writing, p->x, p->y);
+        }
+    }
+
+    return new_writing;
+}
+
 void
 tomoe_writing_move_to (TomoeWriting *writing, gint x, gint y)
 {
