@@ -20,7 +20,7 @@
  *  Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  *  Boston, MA  02111-1307  USA
  *
- *  $Id: tomoe.c,v 1.50 2006/12/14 08:52:57 kous Exp $
+ *  $Id: tomoe.c 1296 2007-06-06 07:35:27Z ktou $
  */
 
 #include <stdio.h>
@@ -44,8 +44,8 @@ tomoe_init (void)
         initialized = TRUE;
         GTypeDebugFlags debug_flag = G_TYPE_DEBUG_NONE;
         g_type_init_with_debug_flags (debug_flag);
-        tomoe_dict_load (NULL);
-        tomoe_recognizer_load (NULL);
+        tomoe_dict_init ();
+        tomoe_recognizer_init ();
     }
 }
 
@@ -60,8 +60,8 @@ tomoe_quit (void)
 {
     if (initialized) {
         initialized = FALSE;
-        tomoe_dict_unload ();
-        tomoe_recognizer_unload ();
+        tomoe_dict_quit ();
+        tomoe_recognizer_quit ();
     }
 }
 
