@@ -17,14 +17,18 @@
  *  Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  *  Boston, MA  02111-1307  USA
  *
- *  $Id: tomoe-dict-est.c,v 1.21 2006/12/27 06:50:46 kous Exp $
+ *  $Id: tomoe-dict-est.c 1493 2007-06-16 10:59:43Z ikezoe $
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif /* HAVE_CONFIG_H */
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <glib.h>
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include <gmodule.h>
 
 #include <estraier.h>
@@ -75,7 +79,7 @@ typedef struct _TomoeDictSearchContext {
 } TomoeDictSearchContext;
 
 static GType tomoe_type_dict_est = 0;
-static GObjectClass *parent_class;
+static TomoeDictClass *parent_class;
 
 static GObject     *constructor               (GType                  type,
                                                guint                  n_props,
@@ -100,6 +104,7 @@ static GList       *search                    (TomoeDict     *dict,
                                                TomoeQuery    *query);
 static gboolean     flush                     (TomoeDict     *dict);
 static gboolean     is_editable               (TomoeDict     *dict);
+static gboolean     is_available              (TomoeDict     *dict);
 static gchar       *get_available_private_utf8 (TomoeDict    *dict);
 static gboolean     tomoe_dict_est_open       (TomoeDictEst  *dict);
 static gboolean     tomoe_dict_est_close      (TomoeDictEst  *dict);
@@ -127,6 +132,7 @@ class_init (TomoeDictEstClass *klass)
     dict_class->search          = search;
     dict_class->flush           = flush;
     dict_class->is_editable     = is_editable;
+    dict_class->is_available    = is_available;
     dict_class->get_available_private_utf8 = get_available_private_utf8;
 
     g_object_class_install_property (
@@ -326,6 +332,7 @@ register_char (TomoeDict *_dict, TomoeChar *chr)
     const gchar *original_value;
     gchar *value;
     gint n_strokes;
+    const GList *readings, *node;
     ESTDOC *doc;
 
     g_return_val_if_fail (TOMOE_IS_DICT_EST (dict), success);
@@ -365,6 +372,54 @@ register_char (TomoeDict *_dict, TomoeChar *chr)
         est_doc_add_attr (doc, "variant", original_value);
     }
 
+    readings = tomoe_char_get_readings (chr);
+    if (readings) {
+        GString *on_readings = NULL, *kun_readings = NULL, *all_readings = NULL;
+        all_readings = g_string_new (NULL);
+        for (node = readings; node; node = g_list_next (node)) {
+            TomoeReading *reading = TOMOE_READING (node->data);
+            const gchar *read_str = tomoe_reading_get_reading (reading);
+            switch (tomoe_reading_get_reading_type (reading)) {
+              case TOMOE_READING_JA_ON:
+                if (on_readings)
+                    on_readings = g_string_append_c (on_readings, ' ');
+                else
+                    on_readings = g_string_new (NULL);
+                on_readings = g_string_append (on_readings, read_str);
+                break;
+              case TOMOE_READING_JA_KUN:
+                if (kun_readings)
+                    kun_readings = g_string_append_c (kun_readings, ' ');
+                else
+                    kun_readings = g_string_new (NULL);
+                kun_readings = g_string_append (kun_readings, read_str);
+                break;
+              case TOMOE_READING_UNKNOWN: 
+              case TOMOE_READING_INVALID: 
+                all_readings = g_string_append_c (all_readings, ' ');
+                all_readings = g_string_append (all_readings, read_str);
+              default:
+                break;
+            }
+        }
+
+        if (on_readings) {
+            est_doc_add_attr (doc, "ja_on_readings", on_readings->str);
+            all_readings = g_string_append_c (all_readings, ' ');
+            all_readings = g_string_append (all_readings, on_readings->str);
+            g_string_free (on_readings, TRUE);
+        }
+        all_readings = g_string_append_c (all_readings, ' ');
+        if (kun_readings) {
+            est_doc_add_attr (doc, "ja_kun_readings", kun_readings->str);
+            all_readings = g_string_append_c (all_readings, ' ');
+            all_readings = g_string_append (all_readings, kun_readings->str);
+            g_string_free (kun_readings, TRUE);
+        }
+        est_doc_add_attr (doc, "all_readings", all_readings->str);
+        g_string_free (all_readings, TRUE);
+    }
+
     value = tomoe_char_to_xml (chr);
     est_doc_add_hidden_text (doc, value);
     g_free (value);
@@ -534,12 +589,39 @@ append_search_cond_readings (TomoeDictEst *dict, ESTCOND *cond, GString *phrase,
 {
     GList *node;
     for (node = (GList *)readings; node; node = g_list_next (node)) {
-        TomoeReading *reading = node->data;
-        gchar *xml;
+        TomoeReading *reading = TOMOE_READING (node->data);
+        const gchar *read_str = tomoe_reading_get_reading (reading);
+        gchar *expr;
 
-        xml = tomoe_reading_to_xml (reading);
-        g_string_append_printf (phrase, " %s", xml);
-        g_free (xml);
+        switch (tomoe_reading_get_reading_type (reading)) {
+#if 0
+          case TOMOE_READING_JA_ON:
+            expr = g_strdup_printf ("ja_on_readings STRINC %s", read_str);
+            est_cond_add_attr (cond, expr);
+            g_free (expr);
+            expr = g_strdup_printf ("all_readings STRINC %s", read_str);
+            est_cond_add_attr (cond, expr);
+            g_free (expr);
+            break;
+          case TOMOE_READING_JA_KUN:
+            expr = g_strdup_printf ("ja_kun_readings STRINC %s", read_str);
+            est_cond_add_attr (cond, expr);
+            g_free (expr);
+            expr = g_strdup_printf ("all_readings STRINC %s", read_str);
+            est_cond_add_attr (cond, expr);
+            g_free (expr);
+            break;
+#endif
+          case TOMOE_READING_JA_ON:
+          case TOMOE_READING_JA_KUN:
+          case TOMOE_READING_INVALID:
+          case TOMOE_READING_UNKNOWN:
+          default:
+            expr = g_strdup_printf ("all_readings STRRX (\\s|^)%s", read_str);
+            est_cond_add_attr (cond, expr);
+            g_free (expr);
+            break;
+        }
     }
 }
 
@@ -630,6 +712,16 @@ is_editable (TomoeDict *_dict)
     return dict->editable;
 }
 
+static gboolean
+is_available (TomoeDict *_dict)
+{
+    TomoeDictEst *dict = TOMOE_DICT_EST (_dict);
+
+    g_return_val_if_fail (TOMOE_IS_DICT_EST (dict), FALSE);
+
+    return dict->db != NULL;
+}
+
 static gchar *
 get_available_private_utf8 (TomoeDict *_dict)
 {
