@@ -17,14 +17,18 @@
  *  Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  *  Boston, MA  02111-1307  USA
  *
- *  $Id: tomoe-dict-mysql.c,v 1.22 2006/12/27 06:50:46 kous Exp $
+ *  $Id: tomoe-dict-mysql.c 1493 2007-06-16 10:59:43Z ikezoe $
  */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif /* HAVE_CONFIG_H */
+
 #include <string.h>
 #include <stdlib.h>
 
 #include <glib.h>
-#include <glib/gi18n.h>
+#include <glib/gi18n-lib.h>
 #include <gmodule.h>
 
 #include <mysql.h>
@@ -90,7 +94,7 @@ struct _RegisterMetaDataContext
 
 
 static GType tomoe_type_dict_mysql = 0;
-static GObjectClass *parent_class;
+static TomoeDictClass *parent_class;
 
 static GObject     *constructor               (GType                  type,
                                                guint                  n_props,
@@ -115,6 +119,7 @@ static GList       *search                    (TomoeDict     *dict,
                                                TomoeQuery    *query);
 static gboolean     flush                     (TomoeDict     *dict);
 static gboolean     is_editable               (TomoeDict     *dict);
+static gboolean     is_available              (TomoeDict     *dict);
 static gchar       *get_available_private_utf8 (TomoeDict    *dict);
 static gboolean     tomoe_dict_mysql_connect  (TomoeDictMySQL  *dict);
 static gboolean     tomoe_dict_mysql_close    (TomoeDictMySQL  *dict);
@@ -142,6 +147,7 @@ class_init (TomoeDictMySQLClass *klass)
     dict_class->search          = search;
     dict_class->flush           = flush;
     dict_class->is_editable     = is_editable;
+    dict_class->is_available    = is_available;
     dict_class->get_available_private_utf8 = get_available_private_utf8;
 
     g_object_class_install_property (
@@ -258,9 +264,9 @@ register_type (GTypeModule *type_module)
         };
 
     tomoe_type_dict_mysql = g_type_module_register_type (type_module,
-                                                       TOMOE_TYPE_DICT,
-                                                       "TomoeDictMySQL",
-                                                       &info, 0);
+                                                         TOMOE_TYPE_DICT,
+                                                         "TomoeDictMySQL",
+                                                         &info, 0);
 }
 
 G_MODULE_EXPORT GList *
@@ -466,6 +472,67 @@ append_string_value (TomoeDictMySQL *dict, GString *sql, const gchar *value)
     g_free (escaped_value);
 }
 
+static gboolean
+does_char_exist (TomoeDictMySQL *dict, const gchar *utf8)
+{
+    GString *sql;
+    gboolean exists = FALSE;
+
+    g_return_val_if_fail (utf8 && *utf8 != '\0', FALSE);
+
+    g_return_val_if_fail (TOMOE_DICT_MYSQL_IS_CONNECTED (dict), FALSE);
+
+    sql = g_string_new ("SELECT utf8 FROM chars WHERE utf8 = ");
+    append_string_value (dict, sql, utf8);
+    if (execute_query (dict, sql->str)) {
+        MYSQL_RES *result;
+        result = mysql_store_result (dict->mysql);
+        exists = mysql_num_rows (result) > 0;
+        mysql_free_result (result);
+        exists = TRUE;
+    }
+
+    g_string_free (sql, TRUE);
+
+    return exists;
+}
+
+static gboolean
+unregister_char_internal (TomoeDictMySQL *dict, const gchar *utf8)
+{
+    GString *sql;
+    gboolean success;
+
+    g_return_val_if_fail (utf8 && *utf8 != '\0', FALSE);
+
+    g_return_val_if_fail (TOMOE_DICT_MYSQL_IS_CONNECTED (dict), FALSE);
+
+    sql = g_string_new ("DELETE FROM chars WHERE utf8 = ");
+    append_string_value (dict, sql, utf8);
+    success = execute_query (dict, sql->str);
+    if (!success) goto done;
+
+    g_string_assign (sql, "DELETE FROM readings WHERE utf8 = ");
+    append_string_value (dict, sql, utf8);
+    success = execute_query (dict, sql->str);
+    if (!success) goto done;
+
+    g_string_assign (sql, "DELETE FROM radicals WHERE utf8 = ");
+    append_string_value (dict, sql, utf8);
+    success = execute_query (dict, sql->str);
+    if (!success) goto done;
+
+    g_string_assign (sql, "DELETE FROM meta_data WHERE utf8 = ");
+    append_string_value (dict, sql, utf8);
+    success = execute_query (dict, sql->str);
+    if (!success) goto done;
+
+  done:
+    g_string_free (sql, TRUE);
+
+    return success;
+}
+
 static gboolean
 register_char_readings (TomoeDictMySQL *dict, const gchar *utf8,
                         const GList *readings)
@@ -570,7 +637,6 @@ static gboolean
 register_char (TomoeDict *_dict, TomoeChar *chr)
 {
     TomoeDictMySQL *dict = TOMOE_DICT_MYSQL (_dict);
-    GString *sql;
     gint n_strokes;
     const gchar *variant;
     const gchar *utf8;
@@ -584,33 +650,39 @@ register_char (TomoeDict *_dict, TomoeChar *chr)
     if (!execute_query (dict, "START TRANSACTION"))
         return FALSE;
 
-    sql = g_string_new ("INSERT INTO chars "
-                        "(utf8, code_point, n_strokes, variant) "
-                        "VALUES (");
-
     utf8 = tomoe_char_get_utf8 (chr);
-    append_string_value (dict, sql, utf8);
+    if (does_char_exist (dict, utf8))
+        success = unregister_char_internal (dict, utf8);
 
-    if (g_utf8_strlen (utf8, -1) == 1)
-        g_string_append_printf (sql, ", %d, ", g_utf8_get_char (utf8));
-    else
-        g_string_append (sql, ", NULL, ");
+    if (success) {
+        GString *sql;
 
-    n_strokes = tomoe_char_get_n_strokes (chr);
-    if (n_strokes >= 0)
-        g_string_append_printf (sql, "%d, ", n_strokes);
-    else
-        g_string_append (sql, "NULL, ");
+        sql = g_string_new ("INSERT INTO chars "
+                            "(utf8, code_point, n_strokes, variant) "
+                            "VALUES (");
+        append_string_value (dict, sql, utf8);
 
-    variant = tomoe_char_get_variant (chr);
-    if (variant)
-        append_string_value (dict, sql, variant);
-    else
-        g_string_append (sql, "NULL");
-    g_string_append_printf (sql, ")");
+        if (g_utf8_strlen (utf8, -1) == 1)
+            g_string_append_printf (sql, ", %d, ", g_utf8_get_char (utf8));
+        else
+            g_string_append (sql, ", NULL, ");
 
-    success = execute_query (dict, sql->str);
-    g_string_free (sql, TRUE);
+        n_strokes = tomoe_char_get_n_strokes (chr);
+        if (n_strokes >= 0)
+            g_string_append_printf (sql, "%d, ", n_strokes);
+        else
+            g_string_append (sql, "NULL, ");
+
+        variant = tomoe_char_get_variant (chr);
+        if (variant)
+            append_string_value (dict, sql, variant);
+        else
+            g_string_append (sql, "NULL");
+        g_string_append_printf (sql, ")");
+
+        success = execute_query (dict, sql->str);
+        g_string_free (sql, TRUE);
+    }
 
     if (success &&
         register_char_readings (dict, utf8, tomoe_char_get_readings (chr)) &&
@@ -627,8 +699,6 @@ static gboolean
 unregister_char (TomoeDict *_dict, const gchar *utf8)
 {
     TomoeDictMySQL *dict = TOMOE_DICT_MYSQL (_dict);
-    GString *sql;
-    gboolean success;
 
     g_return_val_if_fail (TOMOE_IS_DICT_MYSQL (dict), FALSE);
     g_return_val_if_fail (utf8 && *utf8 != '\0', FALSE);
@@ -638,30 +708,7 @@ unregister_char (TomoeDict *_dict, const gchar *utf8)
     if (!execute_query (dict, "START TRANSACTION"))
         return FALSE;
 
-    sql = g_string_new ("DELETE FROM chars WHERE utf8 = ");
-    append_string_value (dict, sql, utf8);
-    success = execute_query (dict, sql->str);
-    if (!success) goto done;
-
-    g_string_assign (sql, "DELETE FROM readings WHERE utf8 = ");
-    append_string_value (dict, sql, utf8);
-    success = execute_query (dict, sql->str);
-    if (!success) goto done;
-
-    g_string_assign (sql, "DELETE FROM radicals WHERE utf8 = ");
-    append_string_value (dict, sql, utf8);
-    success = execute_query (dict, sql->str);
-    if (!success) goto done;
-
-    g_string_assign (sql, "DELETE FROM meta_data WHERE utf8 = ");
-    append_string_value (dict, sql, utf8);
-    success = execute_query (dict, sql->str);
-    if (!success) goto done;
-
-  done:
-    g_string_free (sql, TRUE);
-
-    if (success) {
+    if (unregister_char_internal (dict, utf8)) {
         return execute_query (dict, "COMMIT");
     } else {
         execute_query (dict, "ROLLBACK");
@@ -749,8 +796,11 @@ append_sql_condition_readings (TomoeDictMySQL *dict, GString *sql,
 
         reading = tomoe_reading_get_reading (tomoe_reading);
         if (reading) {
-            g_string_append (sql, "       AND reading = ");
-            append_string_value (dict, sql, reading);
+            gchar *tmp;
+            g_string_append (sql, "       AND reading LIKE ");
+            tmp = g_strdup_printf ("%s%%", reading);
+            append_string_value (dict, sql, tmp);
+            g_free (tmp);
             g_string_append (sql, "\n");
         }
     }
@@ -1028,6 +1078,16 @@ is_editable (TomoeDict *_dict)
     return dict->editable;
 }
 
+static gboolean
+is_available (TomoeDict *_dict)
+{
+    TomoeDictMySQL *dict = TOMOE_DICT_MYSQL (_dict);
+
+    g_return_val_if_fail (TOMOE_IS_DICT_MYSQL (dict), FALSE);
+
+    return dict->mysql != NULL;
+}
+
 static gchar *
 get_available_private_utf8 (TomoeDict *_dict)
 {
