@@ -17,7 +17,7 @@
  *  Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  *  Boston, MA  02111-1307  USA
  *
- *  $Id: tomoe-unihan.h,v 1.1 2006/12/04 04:36:03 kous Exp $
+ *  $Id: tomoe-unihan.h 1490 2007-06-16 10:21:33Z ikezoe $
  */
 
 #ifndef __TOMOE_UNIHAN_H__
@@ -27,7 +27,7 @@
 
 G_BEGIN_DECLS
 
-GPtrArray *_tomoe_unihan_create (void);
+void _tomoe_unihan_create (GPtrArray *array);
 
 G_END_DECLS
 
